@@ -206,16 +206,15 @@ is_online (ExchangeData *ed)
 	session = camel_service_get_session (CAMEL_SERVICE (ed->estore));
 	g_return_val_if_fail (session != NULL, OFFLINE_MODE);
 
-	return camel_session_is_online (session) ? ONLINE_MODE : OFFLINE_MODE;
+	return camel_session_get_online (session) ? ONLINE_MODE : OFFLINE_MODE;
 }
 
 static void
-set_exception (CamelException *ex, const gchar *err)
+set_exception (GError **error, const gchar *err)
 {
 	g_return_if_fail (err != NULL);
 
-	if (ex && !camel_exception_is_set (ex))
-		camel_exception_set (ex, CAMEL_EXCEPTION_SYSTEM, err);
+	g_set_error (error, CAMEL_ERROR, CAMEL_ERROR_GENERIC, "%s", err);
 }
 
 static CamelFolder *
@@ -400,7 +399,7 @@ change_flags (ExchangeFolder *mfld, CamelFolder *folder, ExchangeMessage *mmsg,
 }
 
 static void
-refresh_folder_internal (ExchangeFolder *mfld, CamelException *ex)
+refresh_folder_internal (ExchangeFolder *mfld, GError **error)
 {
 	static const gchar *new_message_props[] = {
 		E2K_PR_REPL_UID,
@@ -527,7 +526,7 @@ refresh_folder_internal (ExchangeFolder *mfld, CamelException *ex)
 
 	if (!E2K_HTTP_STATUS_IS_SUCCESSFUL (status)) {
 		g_warning ("got_new_smtp_messages: %d", status);
-		set_exception (ex, _("Could not get new messages"));
+		set_exception (error, _("Could not get new messages"));
 		goto done;
 	}
 
@@ -566,7 +565,7 @@ refresh_folder_internal (ExchangeFolder *mfld, CamelException *ex)
 
 	if (!E2K_HTTP_STATUS_IS_SUCCESSFUL (status)) {
 		g_warning ("got_new_mapi_messages: %d", status);
-		set_exception (ex, _("Could not get new messages"));
+		set_exception (error, _("Could not get new messages"));
 		goto done;
 	}
 
@@ -760,7 +759,7 @@ sync_deletions (ExchangeFolder *mfld)
 		g_warning ("synced_deleted: %d", status);
 
 	/* Clear out removed messages from mfld */
-	for (my_i = mfld->messages->len - 1; my_i >= 0; my_i --) {
+	for (my_i = mfld->messages->len - 1; my_i >= 0; my_i--) {
 		mmsg = mfld->messages->pdata[my_i];
 		if (!g_hash_table_lookup (known_messages, mmsg)) {
 			mfld->deleted_count++;
@@ -821,9 +820,9 @@ free_folder (gpointer value)
 }
 
 static void
-got_folder_error (ExchangeFolder *mfld, CamelException *ex, const gchar *err)
+got_folder_error (ExchangeFolder *mfld, GError **error, const gchar *err)
 {
-	set_exception (ex, err);
+	set_exception (error, err);
 
 	free_folder (mfld);
 }
@@ -868,7 +867,7 @@ mfld_get_folder_online_sync_updates (gpointer key, gpointer value, gpointer user
 }
 
 static gboolean
-get_folder_contents_online (ExchangeFolder *mfld, CamelException *ex)
+get_folder_contents_online (ExchangeFolder *mfld, GError **error)
 {
 	static const gchar *open_folder_sync_props[] = {
 		E2K_PR_REPL_UID,
@@ -1054,7 +1053,7 @@ get_folder_contents_online (ExchangeFolder *mfld, CamelException *ex)
 	status = e2k_result_iter_free (iter);
 	if (!E2K_HTTP_STATUS_IS_SUCCESSFUL (status)) {
 		g_warning ("got_folder: %d", status);
-		got_folder_error (mfld, ex, _("Could not open folder"));
+		got_folder_error (mfld, error, _("Could not open folder"));
 		return FALSE;
 	}
 
@@ -1181,7 +1180,7 @@ notify_cb (E2kContext *ctx, const gchar *uri, E2kContextChangeType type, gpointe
 }
 
 static gboolean
-get_folder_online (ExchangeFolder *mfld, CamelException *ex)
+get_folder_online (ExchangeFolder *mfld, GError **error)
 {
 	static const gchar *open_folder_props[] = {
 		PR_ACCESS,
@@ -1201,11 +1200,11 @@ get_folder_online (ExchangeFolder *mfld, CamelException *ex)
 					     G_N_ELEMENTS (open_folder_props),
 					     &results, &nresults);
 	if (status == E2K_HTTP_UNAUTHORIZED) {
-		got_folder_error (mfld, ex, _("Could not open folder: Permission denied"));
+		got_folder_error (mfld, error, _("Could not open folder: Permission denied"));
 		return FALSE;
 	} else if (!E2K_HTTP_STATUS_IS_SUCCESSFUL (status)) {
 		g_warning ("got_folder_props: %d", status);
-		got_folder_error (mfld, ex, _("Could not open folder"));
+		got_folder_error (mfld, error, _("Could not open folder"));
 		return FALSE;
 	}
 
@@ -1219,7 +1218,7 @@ get_folder_online (ExchangeFolder *mfld, CamelException *ex)
 		mfld->access = ~0;
 
 	if (!(mfld->access & MAPI_ACCESS_READ)) {
-		got_folder_error (mfld, ex, _("Could not open folder: Permission denied"));
+		got_folder_error (mfld, error, _("Could not open folder: Permission denied"));
 		if (nresults)
 			e2k_results_free (results, nresults);
 		return FALSE;
@@ -1238,7 +1237,7 @@ get_folder_online (ExchangeFolder *mfld, CamelException *ex)
 	   change the PR_LAST_MODIFICATION_TIME property of a message.
 	*/
 	if (g_hash_table_size (mfld->messages_by_href) < 1) {
-		if (!get_folder_contents_online (mfld, ex))
+		if (!get_folder_contents_online (mfld, error))
 			return FALSE;
 	} else {
 		/* FIXME: Pass a GError and handle the error */
@@ -1262,13 +1261,13 @@ get_folder_online (ExchangeFolder *mfld, CamelException *ex)
 }
 
 static ExchangeFolder *
-folder_from_name (ExchangeData *ed, const gchar *folder_name, guint32 perms, CamelException *ex)
+folder_from_name (ExchangeData *ed, const gchar *folder_name, guint32 perms, GError **error)
 {
 	ExchangeFolder *mfld;
 
 	mfld = g_hash_table_lookup (ed->folders_by_name, folder_name);
 	if (!mfld) {
-		set_exception (ex, _("No such folder"));
+		set_exception (error, _("No such folder"));
 		return NULL;
 	}
 
@@ -1290,7 +1289,7 @@ folder_from_name (ExchangeData *ed, const gchar *folder_name, guint32 perms, Cam
 	}
 
 	if (perms && !(mfld->access & perms)) {
-		set_exception (ex, _("Permission denied"));
+		set_exception (error, _("Permission denied"));
 		return NULL;
 	}
 
@@ -1891,7 +1890,7 @@ foreign_new_folder_cb (ExchangeAccount *account, EFolder *folder, GPtrArray *fol
 }
 
 static void
-get_folder_info_data (ExchangeData *ed, const gchar *top, guint32 store_flags, GPtrArray **names, GPtrArray **uris, GArray **unread, GArray **flags)
+get_folder_info_data (ExchangeData *ed, const gchar *top, guint32 store_flags, GHashTable *known_uris, GPtrArray **names, GPtrArray **uris, GArray **unread, GArray **flags)
 {
 	GPtrArray *folders = NULL;
 	ExchangeHierarchy *hier;
@@ -1902,6 +1901,7 @@ get_folder_info_data (ExchangeData *ed, const gchar *top, guint32 store_flags, G
 	gboolean recursive, subscribed, subscription_list;
 	gint mode = -1;
 	gchar *full_path;
+	GSList *check_children = NULL;
 
 	recursive = (store_flags & CAMEL_STORE_FOLDER_INFO_RECURSIVE);
 	subscribed = (store_flags & CAMEL_STORE_FOLDER_INFO_SUBSCRIBED);
@@ -1934,10 +1934,14 @@ get_folder_info_data (ExchangeData *ed, const gchar *top, guint32 store_flags, G
 		folders = exchange_account_get_folders (ed->account);
 	}
 
-	*names = g_ptr_array_new ();
-	*uris = g_ptr_array_new ();
-	*unread = g_array_new (FALSE, FALSE, sizeof (gint));
-	*flags = g_array_new (FALSE, FALSE, sizeof (gint));
+	if (!*names)
+		*names = g_ptr_array_new ();
+	if (!*uris)
+		*uris = g_ptr_array_new ();
+	if (!*unread)
+		*unread = g_array_new (FALSE, FALSE, sizeof (gint));
+	if (!*flags)
+		*flags = g_array_new (FALSE, FALSE, sizeof (gint));
 	/* Can be NULL if started in offline mode */
 	if (ed->inbox) {
 		inbox_uri = e_folder_get_physical_uri (ed->inbox);
@@ -2030,12 +2034,22 @@ get_folder_info_data (ExchangeData *ed, const gchar *top, guint32 store_flags, G
 			if (!e_folder_exchange_get_has_subfolders (folder)) {
 				d(printf ("%s:%d:%s - %s has no subfolders", __FILE__, __LINE__, G_STRFUNC, name));
 				folder_flags |= CAMEL_FOLDER_NOCHILDREN;
+			} else if (recursive && !subscribed && subscription_list && known_uris && g_hash_table_lookup (known_uris, uri) == NULL) {
+				gchar *path = strrchr (uri, ';');
+				if (path && g_ascii_strcasecmp (path + 1, "public") != 0) {
+					path = g_uri_unescape_string (path + 1, NULL);
+					check_children = g_slist_prepend (check_children, path);
+				}
 			}
 
 			d(g_print ("folder flags is : %d\n", folder_flags));
 
+			uri = g_strdup (uri);
+			if (known_uris)
+				g_hash_table_insert (known_uris, (gchar *) uri, GINT_TO_POINTER (1));
+
 			g_ptr_array_add (*names, g_strdup (name));
-			g_ptr_array_add (*uris, g_strdup (uri));
+			g_ptr_array_add (*uris, (gchar *)uri);
 			g_array_append_val (*unread, unread_count);
 			g_array_append_val (*flags, folder_flags);
 		}
@@ -2044,13 +2058,25 @@ get_folder_info_data (ExchangeData *ed, const gchar *top, guint32 store_flags, G
 			g_signal_handler_disconnect (ed->account, new_folder_handler_id);
 		g_ptr_array_free (folders, TRUE);
 	}
+
+	if (check_children) {
+		GSList *l;
+
+		check_children = g_slist_reverse (check_children);
+		for (l = check_children; l; l = l->next) {
+			get_folder_info_data (ed, l->data, store_flags, known_uris, names, uris, unread, flags);
+		}
+
+		g_slist_foreach (check_children, (GFunc) g_free, NULL);
+		g_slist_free (check_children);
+	}
 }
 
 struct update_linestatus
 {
 	CamelExchangeStore *estore;
 	gint linestatus;
-	CamelException *ex;
+	GError **error;
 };
 
 static void
@@ -2065,7 +2091,7 @@ folder_update_linestatus (gpointer key, gpointer value, gpointer data)
 	if (ul->linestatus == ONLINE_MODE) {
 		CamelFolder *folder;
 
-		get_folder_online (mfld, ul->ex);
+		get_folder_online (mfld, ul->error);
 
 		readonly = (mfld->access & (MAPI_ACCESS_MODIFY | MAPI_ACCESS_CREATE_CONTENTS)) ? 0 : 1;
 
@@ -2082,7 +2108,7 @@ gboolean
 camel_exchange_utils_connect (CamelService *service,
 				const gchar *pwd,
 				guint32 *status, /* out */
-				CamelException *ex)
+				GError **error)
 {
 	ExchangeData *ed = get_data_for_service (service);
 	ExchangeAccount *account;
@@ -2092,6 +2118,12 @@ camel_exchange_utils_connect (CamelService *service,
 	const gchar *uri;
 	struct update_linestatus ul;
 
+	if (ed == NULL) {
+		g_set_error (error, CAMEL_ERROR, CAMEL_ERROR_GENERIC,
+			"Could not find Exchange account. Make sure you've only one Exchange account configured.");
+		return FALSE;
+	}
+
 	g_return_val_if_fail (ed != NULL, FALSE);
 	g_return_val_if_fail (status != NULL, FALSE);
 
@@ -2129,7 +2161,7 @@ camel_exchange_utils_connect (CamelService *service,
 	   the first time */
 
 	ul.estore = ed->estore;
-	ul.ex = ex;
+	ul.error = error;
 	g_hash_table_foreach (ed->folders_by_name,
 			      (GHFunc) folder_update_linestatus,
 			      &ul);
@@ -2150,7 +2182,7 @@ camel_exchange_utils_get_folder (CamelService *service,
 				guint32 *folder_flags, /* out */
 				gchar **folder_uri, /* out */
 				gboolean *readonly, /* out */
-				CamelException *ex)
+				GError **error)
 {
 	ExchangeData *ed = get_data_for_service (service);
 	ExchangeFolder *mfld;
@@ -2170,7 +2202,7 @@ camel_exchange_utils_get_folder (CamelService *service,
 	path = g_strdup_printf ("/%s", name);
 	folder = exchange_account_get_folder (ed->account, path);
 	if (!folder && !create) {
-		set_exception (ex, _("No such folder"));
+		set_exception (error, _("No such folder"));
 		g_free (path);
 		return FALSE;
 	} else if (!folder) {
@@ -2179,7 +2211,7 @@ camel_exchange_utils_get_folder (CamelService *service,
 		result = exchange_account_create_folder (ed->account, path, "mail");
 		folder = exchange_account_get_folder (ed->account, path);
 		if (result != EXCHANGE_ACCOUNT_FOLDER_OK || !folder) {
-			set_exception (ex, _("Could not create folder."));
+			set_exception (error, _("Could not create folder."));
 			g_free (path);
 			return FALSE;
 		}
@@ -2227,9 +2259,8 @@ camel_exchange_utils_get_folder (CamelService *service,
 	mfld->high_article_num = high_article_num;
 
 	if (is_online (ed) == ONLINE_MODE) {
-		if (!get_folder_online (mfld, ex)) {
+		if (!get_folder_online (mfld, error))
 			return FALSE;
-		}
 	}
 	g_signal_connect (mfld->folder, "changed",
 			  G_CALLBACK (storage_folder_changed), mfld);
@@ -2260,7 +2291,7 @@ camel_exchange_utils_get_folder (CamelService *service,
 gboolean
 camel_exchange_utils_get_trash_name (CamelService *service,
 				gchar **trash_name, /* out */
-				CamelException *ex)
+				GError **error)
 {
 	ExchangeData *ed = get_data_for_service (service);
 
@@ -2268,7 +2299,7 @@ camel_exchange_utils_get_trash_name (CamelService *service,
 	g_return_val_if_fail (trash_name != NULL, FALSE);
 
 	if (!ed->deleted_items) {
-		set_exception (ex, _("Could not open Deleted Items folder"));
+		set_exception (error, _("Could not open Deleted Items folder"));
 		return FALSE;
 	}
 
@@ -2280,19 +2311,18 @@ camel_exchange_utils_get_trash_name (CamelService *service,
 gboolean
 camel_exchange_utils_refresh_folder (CamelService *service,
 				const gchar *folder_name,
-				CamelException *ex)
+				GError **error)
 {
 	ExchangeData *ed = get_data_for_service (service);
 	ExchangeFolder *mfld;
 
 	g_return_val_if_fail (ed != NULL, FALSE);
 
-	mfld = folder_from_name (ed, folder_name, 0, ex);
-	if (!mfld) {
+	mfld = folder_from_name (ed, folder_name, 0, error);
+	if (!mfld)
 		return FALSE;
-	}
 
-	refresh_folder_internal (mfld, ex);
+	refresh_folder_internal (mfld, NULL);
 	sync_deletions (mfld);
 
 	return TRUE;
@@ -2303,7 +2333,7 @@ camel_exchange_utils_sync_count (CamelService *service,
 				const gchar *folder_name,
 				guint32 *unread_count, /* out */
 				guint32 *visible_count, /* out */
-				CamelException *ex)
+				GError **error)
 {
 	ExchangeData *ed = get_data_for_service (service);
 	ExchangeFolder *mfld;
@@ -2312,7 +2342,7 @@ camel_exchange_utils_sync_count (CamelService *service,
 	g_return_val_if_fail (unread_count != NULL, FALSE);
 	g_return_val_if_fail (visible_count != NULL, FALSE);
 
-	mfld = folder_from_name (ed, folder_name, 0, ex);
+	mfld = folder_from_name (ed, folder_name, 0, error);
 	if (mfld) {
 		*unread_count = mfld->unread_count;
 		*visible_count = mfld->messages->len;
@@ -2328,7 +2358,7 @@ gboolean
 camel_exchange_utils_expunge_uids (CamelService *service,
 				const gchar *folder_name,
 				GPtrArray *uids,
-				CamelException *ex)
+				GError **error)
 {
 	ExchangeData *ed = get_data_for_service (service);
 	ExchangeFolder *mfld;
@@ -2346,7 +2376,7 @@ camel_exchange_utils_expunge_uids (CamelService *service,
 	if (!uids->len)
 		return TRUE;
 
-	mfld = folder_from_name (ed, folder_name, MAPI_ACCESS_DELETE, ex);
+	mfld = folder_from_name (ed, folder_name, MAPI_ACCESS_DELETE, error);
 	if (!mfld)
 		return FALSE;
 
@@ -2395,9 +2425,9 @@ camel_exchange_utils_expunge_uids (CamelService *service,
 	if (!E2K_HTTP_STATUS_IS_SUCCESSFUL (status)) {
 		g_warning ("expunged: %d", status);
 		some_error = TRUE;
-		set_exception (ex, _("Could not empty Deleted Items folder"));
+		set_exception (error, _("Could not empty Deleted Items folder"));
 	} else if (some_error) {
-		set_exception (ex, _("Permission denied. Could not delete certain messages."));
+		set_exception (error, _("Permission denied. Could not delete certain messages."));
 	}
 
 	g_ptr_array_free (hrefs, TRUE);
@@ -2418,7 +2448,7 @@ camel_exchange_utils_append_message (CamelService *service,
 				const gchar *subject,
 				const GByteArray *message,
 				gchar **new_uid, /* out */
-				CamelException *ex)
+				GError **error)
 {
 	ExchangeData *ed = get_data_for_service (service);
 	ExchangeFolder *mfld;
@@ -2429,7 +2459,8 @@ camel_exchange_utils_append_message (CamelService *service,
 	g_return_val_if_fail (new_uid != NULL, FALSE);
 	g_return_val_if_fail (message != NULL, FALSE);
 
-	mfld = folder_from_name (ed, folder_name, MAPI_ACCESS_CREATE_CONTENTS, ex);
+	mfld = folder_from_name (
+		ed, folder_name, MAPI_ACCESS_CREATE_CONTENTS, error);
 	if (!mfld)
 		return FALSE;
 
@@ -2439,7 +2470,7 @@ camel_exchange_utils_append_message (CamelService *service,
 					    &location, &ru_header);
 	if (status != E2K_HTTP_CREATED) {
 		g_warning ("appended_message: %d", status);
-		set_exception (ex, status == E2K_HTTP_INSUFFICIENT_SPACE_ON_RESOURCE ?
+		set_exception (error, status == E2K_HTTP_INSUFFICIENT_SPACE_ON_RESOURCE ?
 				   _("Could not append message; mailbox is over quota") :
 				   _("Could not append message"));
 		return FALSE;
@@ -2495,7 +2526,7 @@ camel_exchange_utils_set_message_flags (CamelService *service,
 					const gchar *uid,
 					guint32 flags,
 					guint32 mask,
-					CamelException *ex)
+					GError **error)
 {
 	ExchangeData *ed = get_data_for_service (service);
 	ExchangeFolder *mfld;
@@ -2503,7 +2534,7 @@ camel_exchange_utils_set_message_flags (CamelService *service,
 
 	g_return_val_if_fail (ed != NULL, FALSE);
 
-	mfld = folder_from_name (ed, folder_name, MAPI_ACCESS_MODIFY, ex);
+	mfld = folder_from_name (ed, folder_name, MAPI_ACCESS_MODIFY, error);
 	if (!mfld)
 		return FALSE;
 
@@ -2571,7 +2602,7 @@ camel_exchange_utils_set_message_tag (CamelService *service,
 				const gchar *uid,
 				const gchar *name,
 				const gchar *value,
-				CamelException *ex)
+				GError **error)
 {
 	ExchangeData *ed = get_data_for_service (service);
 	ExchangeFolder *mfld;
@@ -2579,7 +2610,7 @@ camel_exchange_utils_set_message_tag (CamelService *service,
 
 	g_return_val_if_fail (ed != NULL, FALSE);
 
-	mfld = folder_from_name (ed, folder_name, MAPI_ACCESS_MODIFY, ex);
+	mfld = folder_from_name (ed, folder_name, MAPI_ACCESS_MODIFY, error);
 	if (!mfld)
 		return FALSE;
 
@@ -2599,7 +2630,7 @@ camel_exchange_utils_get_message (CamelService *service,
 				const gchar *folder_name,
 				const gchar *uid,
 				GByteArray **message_bytes, /* out */
-				CamelException *ex)
+				GError **error)
 {
 	ExchangeData *ed = get_data_for_service (service);
 	ExchangeFolder *mfld;
@@ -2614,7 +2645,7 @@ camel_exchange_utils_get_message (CamelService *service,
 	g_return_val_if_fail (ed != NULL, FALSE);
 	g_return_val_if_fail (message_bytes != NULL, FALSE);
 
-	mfld = folder_from_name (ed, folder_name, MAPI_ACCESS_READ, ex);
+	mfld = folder_from_name (ed, folder_name, MAPI_ACCESS_READ, error);
 	if (!mfld)
 		return FALSE;
 
@@ -2627,7 +2658,7 @@ camel_exchange_utils_get_message (CamelService *service,
 			camel_exchange_folder_remove_message (CAMEL_EXCHANGE_FOLDER (folder), uid);
 		}
 
-		set_exception (ex, _("No such message"));
+		set_exception (error, _("No such message"));
 		return FALSE;
 	}
 
@@ -2699,9 +2730,9 @@ camel_exchange_utils_get_message (CamelService *service,
 		 * time we recorded that.
 		 */
 		message_removed (mfld, folder, mmsg->href);
-		set_exception (ex, _("Message has been deleted"));
+		set_exception (error, _("Message has been deleted"));
 	} else
-		set_exception (ex, _("Error retrieving message"));
+		set_exception (error, _("Error retrieving message"));
 
  cleanup:
 	g_free (body);
@@ -2716,7 +2747,7 @@ camel_exchange_utils_search (CamelService *service,
 				const gchar *folder_name,
 				const gchar *text,
 				GPtrArray **found_uids, /* out */
-				CamelException *ex)
+				GError **error)
 {
 	ExchangeData *ed = get_data_for_service (service);
 	ExchangeFolder *mfld;
@@ -2730,7 +2761,7 @@ camel_exchange_utils_search (CamelService *service,
 	g_return_val_if_fail (ed != NULL, FALSE);
 	g_return_val_if_fail (found_uids != NULL, FALSE);
 
-	mfld = folder_from_name (ed, folder_name, 0, ex);
+	mfld = folder_from_name (ed, folder_name, 0, error);
 	if (!mfld)
 		return FALSE;
 
@@ -2750,7 +2781,7 @@ camel_exchange_utils_search (CamelService *service,
 	status = e2k_result_iter_free (iter);
 
 	if (status == E2K_HTTP_UNPROCESSABLE_ENTITY) {
-		set_exception (ex, _("Mailbox does not support full-text searching"));
+		set_exception (error, _("Mailbox does not support full-text searching"));
 
 		g_ptr_array_foreach (matches, (GFunc) g_free, NULL);
 		g_ptr_array_free (matches, TRUE);
@@ -2769,7 +2800,7 @@ camel_exchange_utils_transfer_messages (CamelService *service,
 					GPtrArray *uids,
 					gboolean delete_originals,
 					GPtrArray **ret_uids, /* out */
-					CamelException *ex)
+					GError **error)
 {
 	ExchangeData *ed = get_data_for_service (service);
 	ExchangeFolder *source, *dest;
@@ -2787,10 +2818,14 @@ camel_exchange_utils_transfer_messages (CamelService *service,
 	g_return_val_if_fail (ed != NULL, FALSE);
 	g_return_val_if_fail (ret_uids != NULL, FALSE);
 
-	source = folder_from_name (ed, source_name, delete_originals ? MAPI_ACCESS_DELETE : 0, ex);
+	source = folder_from_name (
+		ed, source_name, delete_originals ?
+		MAPI_ACCESS_DELETE : 0, error);
 	if (!source)
 		return FALSE;
-	dest = folder_from_name (ed, dest_name, MAPI_ACCESS_CREATE_CONTENTS, ex);
+
+	dest = folder_from_name (
+		ed, dest_name, MAPI_ACCESS_CREATE_CONTENTS, error);
 	if (!dest)
 		return FALSE;
 
@@ -2858,7 +2893,7 @@ camel_exchange_utils_transfer_messages (CamelService *service,
 		*ret_uids = new_uids;
 	} else {
 		g_warning ("transferred_messages: %d", status);
-		set_exception (ex, _("Unable to move/copy messages"));
+		set_exception (error, _("Unable to move/copy messages"));
 		g_ptr_array_free (new_uids, TRUE);
 		new_uids = NULL;
 	}
@@ -2927,9 +2962,10 @@ camel_exchange_utils_get_folder_info (CamelService *service,
 					GPtrArray **folder_uris, /* out */
 					GArray **unread_counts, /* out */
 					GArray **folder_flags, /* out */
-					CamelException *ex)
+					GError **error)
 {
 	ExchangeData *ed = get_data_for_service (service);
+	GHashTable *known_uris;
 
 	g_return_val_if_fail (ed != NULL, FALSE);
 	g_return_val_if_fail (folder_names != NULL, FALSE);
@@ -2942,7 +2978,15 @@ camel_exchange_utils_get_folder_info (CamelService *service,
 	   from more than one thread */
 	g_static_rec_mutex_lock (&ed->changed_msgs_mutex);
 
-	get_folder_info_data (ed, top, store_flags, folder_names, folder_uris, unread_counts, folder_flags);
+	*folder_names = NULL;
+	*folder_uris = NULL;
+	*unread_counts = NULL;
+	*folder_flags = NULL;
+
+	/* hash table of known uris. The uri key is shared with folder_uris, thus no need to free it */
+	known_uris = g_hash_table_new (g_str_hash, g_str_equal);
+	get_folder_info_data (ed, top, store_flags, known_uris, folder_names, folder_uris, unread_counts, folder_flags);
+	g_hash_table_destroy (known_uris);
 
 	if (ed->new_folder_id == 0) {
 		ed->new_folder_id = g_signal_connect (ed->account, "new_folder", G_CALLBACK (account_new_folder), ed);
@@ -2959,7 +3003,7 @@ camel_exchange_utils_send_message (CamelService *service,
 				const gchar *from,
 				GPtrArray *recipients,
 				const GByteArray *message,
-				CamelException *ex)
+				GError **error)
 {
 	ExchangeData *ed = get_data_for_service (service);
 	SoupMessage *msg;
@@ -2977,7 +3021,7 @@ camel_exchange_utils_send_message (CamelService *service,
 	g_return_val_if_fail (ed->estore != NULL, FALSE);
 
 	if (!ed->mail_submission_uri) {
-		set_exception (ex, _("No mail submission URI for this mailbox"));
+		set_exception (error, _("No mail submission URI for this mailbox"));
 		return FALSE;
 	}
 
@@ -3010,12 +3054,12 @@ camel_exchange_utils_send_message (CamelService *service,
 	if (E2K_HTTP_STATUS_IS_SUCCESSFUL (status)) {
 		res = TRUE;
 	} else if (status == E2K_HTTP_NOT_FOUND) {
-		set_exception (ex, _("Server won't accept mail via Exchange transport"));
+		set_exception (error, _("Server won't accept mail via Exchange transport"));
 	} else if (status == E2K_HTTP_FORBIDDEN) {
 		errmsg = g_strdup_printf (_("Your account does not have permission "
 					    "to use <%s>\nas a From address."),
 					  from);
-		set_exception (ex, errmsg);
+		set_exception (error, errmsg);
 		g_free (errmsg);
 	} else if (status == E2K_HTTP_INSUFFICIENT_SPACE_ON_RESOURCE ||
 		   status == E2K_HTTP_INTERNAL_SERVER_ERROR) {
@@ -3024,11 +3068,11 @@ camel_exchange_utils_send_message (CamelService *service,
 		 * changes in the future.)
 		 */
 		E2K_KEEP_PRECEDING_COMMENT_OUT_OF_PO_FILES;
-		set_exception (ex, _("Could not send message.\n"
+		set_exception (error, _("Could not send message.\n"
 				     "This might mean that your account is over quota."));
 	} else {
 		g_warning ("sent_message: %d", status);
-		set_exception (ex, _("Could not send message"));
+		set_exception (error, _("Could not send message"));
 	}
 
 	return res;
@@ -3041,7 +3085,7 @@ camel_exchange_utils_create_folder (CamelService *service,
 				gchar **folder_uri, /* out */
 				guint32 *unread_count, /* out */
 				guint32 *flags, /* out */
-				CamelException *ex)
+				GError **error)
 {
 	ExchangeData *ed = get_data_for_service (service);
 	ExchangeAccountFolderResult result;
@@ -3064,15 +3108,15 @@ camel_exchange_utils_create_folder (CamelService *service,
 			break;
 		/* fall through */
 	default:
-		set_exception (ex, _("Generic error"));
+		set_exception (error, _("Generic error"));
 		return FALSE;
 
 	case EXCHANGE_ACCOUNT_FOLDER_ALREADY_EXISTS:
-		set_exception (ex, _("Folder already exists"));
+		set_exception (error, _("Folder already exists"));
 		return FALSE;
 
 	case EXCHANGE_ACCOUNT_FOLDER_PERMISSION_DENIED:
-		set_exception (ex, _("Permission denied"));
+		set_exception (error, _("Permission denied"));
 		return FALSE;
 	}
 
@@ -3086,7 +3130,7 @@ camel_exchange_utils_create_folder (CamelService *service,
 gboolean
 camel_exchange_utils_delete_folder (CamelService *service,
 				const gchar *folder_name,
-				CamelException *ex)
+				GError **error)
 {
 	ExchangeData *ed = get_data_for_service (service);
 	ExchangeAccountFolderResult result;
@@ -3098,7 +3142,7 @@ camel_exchange_utils_delete_folder (CamelService *service,
 	path = g_build_filename ("/", folder_name, NULL);
 	folder = exchange_account_get_folder (ed->account, path);
 	if (!folder) {
-		set_exception (ex, _("Folder doesn't exist"));
+		set_exception (error, _("Folder doesn't exist"));
 		g_free (path);
 		return FALSE;
 	}
@@ -3115,12 +3159,12 @@ camel_exchange_utils_delete_folder (CamelService *service,
 
 	case EXCHANGE_ACCOUNT_FOLDER_PERMISSION_DENIED: /* Fall through */
 	case EXCHANGE_ACCOUNT_FOLDER_UNSUPPORTED_OPERATION:
-		set_exception (ex, _("Permission denied"));
+		set_exception (error, _("Permission denied"));
 		g_object_unref (folder);
 		return FALSE;
 
 	default:
-		set_exception (ex, _("Generic error"));
+		set_exception (error, _("Generic error"));
 		g_object_unref (folder);
 		return FALSE;
 
@@ -3139,15 +3183,15 @@ camel_exchange_utils_rename_folder (CamelService *service,
 				GPtrArray **folder_uris, /* out */
 				GArray **unread_counts, /* out */
 				GArray **folder_flags, /* out */
-				CamelException *ex)
+				GError **error)
 {
 	ExchangeData *ed = get_data_for_service (service);
 	ExchangeFolder *mfld;
 	ExchangeAccountFolderResult result;
 	EFolder *folder;
 	gchar *old_path, *new_path;
-	GPtrArray *names, *uris;
-	GArray *unread, *flags;
+	GPtrArray *names = NULL, *uris = NULL;
+	GArray *unread = NULL, *flags = NULL;
 	gint i = 0, j = 0;
 	gchar **folder_name;
 	const gchar *uri;
@@ -3162,7 +3206,7 @@ camel_exchange_utils_rename_folder (CamelService *service,
 	old_path = g_build_filename ("/", old_name, NULL);
 	folder = exchange_account_get_folder (ed->account, old_path);
 	if (!folder) {
-		set_exception (ex, _("Folder doesn't exist"));
+		set_exception (error, _("Folder doesn't exist"));
 		g_free (old_path);
 		return FALSE;
 	}
@@ -3189,7 +3233,7 @@ camel_exchange_utils_rename_folder (CamelService *service,
 		g_hash_table_steal (ed->folders_by_name, old_name);
 		g_hash_table_insert (ed->folders_by_name, (gchar *)mfld->name, mfld);
 
-		get_folder_info_data (ed, new_name, CAMEL_STORE_FOLDER_INFO_SUBSCRIBED, &names, &uris, &unread, &flags);
+		get_folder_info_data (ed, new_name, CAMEL_STORE_FOLDER_INFO_SUBSCRIBED, NULL, &names, &uris, &unread, &flags);
 
 		g_hash_table_remove_all (mfld->messages_by_href);
 
@@ -3201,9 +3245,8 @@ camel_exchange_utils_rename_folder (CamelService *service,
 		}
 
 		if (is_online (ed) == ONLINE_MODE) {
-			if (!get_folder_online (mfld, ex)) {
+			if (!get_folder_online (mfld, error))
 				return FALSE;
-			}
 		}
 
 		for (i = 0; i < uris->len; i++) {
@@ -3263,9 +3306,8 @@ camel_exchange_utils_rename_folder (CamelService *service,
 			}
 
 			if (is_online (ed) == ONLINE_MODE) {
-				if (!get_folder_online (mfld, ex)) {
+				if (!get_folder_online (mfld, error))
 					return FALSE;
-				}
 			}
 
 			g_free (old_path);
@@ -3282,15 +3324,15 @@ camel_exchange_utils_rename_folder (CamelService *service,
 		break;
 
 	case EXCHANGE_ACCOUNT_FOLDER_DOES_NOT_EXIST:
-		set_exception (ex, _("Folder doesn't exist"));
+		set_exception (error, _("Folder doesn't exist"));
 		return FALSE;
 
 	case EXCHANGE_ACCOUNT_FOLDER_PERMISSION_DENIED:
-		set_exception (ex, _("Permission denied"));
+		set_exception (error, _("Permission denied"));
 		return FALSE;
 
 	default:
-		set_exception (ex, _("Generic error"));
+		set_exception (error, _("Generic error"));
 		return FALSE;
 
 	}
@@ -3301,7 +3343,7 @@ camel_exchange_utils_rename_folder (CamelService *service,
 gboolean
 camel_exchange_utils_subscribe_folder (CamelService *service,
 					const gchar *folder_name,
-					CamelException *ex)
+					GError **error)
 {
 	ExchangeData *ed = get_data_for_service (service);
 	ExchangeAccountFolderResult result;
@@ -3313,7 +3355,7 @@ camel_exchange_utils_subscribe_folder (CamelService *service,
 	path = g_build_filename ("/", folder_name, NULL);
 	folder = exchange_account_get_folder (ed->account, path);
 	if (!folder) {
-		set_exception (ex, _("Folder doesn't exist"));
+		set_exception (error, _("Folder doesn't exist"));
 		g_free (path);
 		return FALSE;
 	}
@@ -3339,11 +3381,11 @@ camel_exchange_utils_subscribe_folder (CamelService *service,
 		break;
 
 	case EXCHANGE_ACCOUNT_FOLDER_PERMISSION_DENIED:
-		set_exception (ex, _("Permission denied"));
+		set_exception (error, _("Permission denied"));
 		return FALSE;
 
 	default:
-		set_exception (ex, _("Generic error"));
+		set_exception (error, _("Generic error"));
 		return FALSE;
 	}
 
@@ -3353,7 +3395,7 @@ camel_exchange_utils_subscribe_folder (CamelService *service,
 gboolean
 camel_exchange_utils_unsubscribe_folder (CamelService *service,
 					const gchar *folder_name,
-					CamelException *ex)
+					GError **error)
 {
 	ExchangeData *ed = get_data_for_service (service);
 	ExchangeAccountFolderResult result;
@@ -3365,7 +3407,7 @@ camel_exchange_utils_unsubscribe_folder (CamelService *service,
 	path = g_build_filename ("/", folder_name, NULL);
 	folder = exchange_account_get_folder (ed->account, path);
 	if (!folder) {
-		set_exception (ex, _("Folder doesn't exist"));
+		set_exception (error, _("Folder doesn't exist"));
 		g_free (path);
 		return FALSE;
 	}
@@ -3386,7 +3428,7 @@ camel_exchange_utils_unsubscribe_folder (CamelService *service,
 	path = g_build_filename ("/favorites", pub_name, NULL);
 	folder = exchange_account_get_folder (ed->account, path);
 	if (!folder) {
-		set_exception (ex, _("Folder doesn't exist"));
+		set_exception (error, _("Folder doesn't exist"));
 		g_free (path);
 		return FALSE;
 	}
@@ -3401,13 +3443,13 @@ camel_exchange_utils_unsubscribe_folder (CamelService *service,
 		break;
 
 	case EXCHANGE_ACCOUNT_FOLDER_PERMISSION_DENIED:
-		set_exception (ex, _("Permission denied"));
+		set_exception (error, _("Permission denied"));
 		g_object_unref (folder);
 		g_free (path);
 		return FALSE;
 
 	default:
-		set_exception (ex, _("Generic error"));
+		set_exception (error, _("Generic error"));
 		g_object_unref (folder);
 		g_free (path);
 		return FALSE;
@@ -3424,7 +3466,7 @@ gboolean
 camel_exchange_utils_is_subscribed_folder (CamelService *service,
 					const gchar *folder_name,
 					gboolean *is_subscribed, /* out */
-					CamelException *ex)
+					GError **error)
 {
 	ExchangeData *ed = get_data_for_service (service);
 	EFolder *folder;
